/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef _ONDEVICESPEECHENGINE_H_
#define _ONDEVICESPEECHENGINE_H_

#include <kylin-ai/ai-engine/large-model/endsidespeechengine.h>
#include <kylin-ai/ai-engine/large-model/error.h>

namespace ai_engine::lm::speech {
class OndeviceSpeechEngine final : public AbstractEndSideSpeechEngine {
public:
    OndeviceSpeechEngine() = default;
    ~OndeviceSpeechEngine() override = default;

    std::string backendName() override { return "ondevice-speech"; }

    void setRecognizingCallback(
        ai_engine::lm::speech::RecognitionResultCallback callback) override;
    void setRecognizedCallback(
        ai_engine::lm::speech::RecognitionResultCallback callback);

    void setSynthesizingCallback(
        ai_engine::lm::speech::SynthesisResultCallback callback) override;
    void setSynthesizedCallback(
        ai_engine::lm::speech::FinishedCallback callback) override;

    // 流式语音识别
    bool
    initContinuousRecognitionModule(ai_engine::lm::EngineError &error) override;
    bool startContinuousRecognition(const std::string &params,
                                    ai_engine::lm::EngineError &error) override;
    bool writeContinuousRecognitionAudioData(
        const std::vector<char> &audioData,
        ai_engine::lm::EngineError &error) override;
    bool stopContinuousRecognition(ai_engine::lm::EngineError &error) override;
    bool destroyContinuousRecognitionModule(
        ai_engine::lm::EngineError &error) override;

    // 非流式语音识别
    bool initRecognizeOnceModule(ai_engine::lm::EngineError &error) override;
    bool recognizeOnce(const std::string &params,
                       const std::vector<char> &audioData,
                       ai_engine::lm::EngineError &error) override;
    bool destroyRecognizeOnceModule(ai_engine::lm::EngineError &error) override;

    // 流式语音合成
    bool
    initContinuousSynthesisModule(ai_engine::lm::EngineError &error) override;
    bool startContinuousSynthesis(const std::string &params,
                                  ai_engine::lm::EngineError &error) override;
    bool
    writeContinuousSynthesisText(const std::string &text,
                                 ai_engine::lm::EngineError &error) override;
    bool stopContinuousSynthesis(ai_engine::lm::EngineError &error) override;
    bool destroyContinuousSynthesisModule(
        ai_engine::lm::EngineError &error) override;

    // 非流式语音合成
    bool initSynthesizeOnceModule(ai_engine::lm::EngineError &error) override;
    bool synthesizeOnce(const std::string &params, const std::string &text,
                        ai_engine::lm::EngineError &error) override;
    bool
    destroySynthesizeOnceModule(ai_engine::lm::EngineError &error) override;

    // 关键字识别
    bool initContinuousKeywordRecognitionModule(EngineError &error) override;
    bool startContinuousKeywordRecognition(const std::string &params,
                                           EngineError &error) override;
    bool writeContinuousKeywordRecognitionAudioData(
        const std::vector<char> &audioData, EngineError &error) override;
    bool stopContinuousKeywordRecognition(EngineError &error) override;
    bool destroyContinuousKeywordRecognitionModule(EngineError &error) override;

private:
    uint sessionId_ = 0;
    uint taskid_ = 0;
    std::string continuousSynthesisParams_;
};
} // namespace ai_engine::lm::speech

#endif
